<?php

/**
 * Client for querying fonecta interface
 */
class Fonecta {

  /**
   * Fonecta interface message codes
   */
  const MESSAGE_CODE_ERROR_TOO_MANY_RESULTS = 'ER9100';
  const MESSAGE_CODE_ERROR_NOTHING_FOUND = 'ER9090';
  const MESSAGE_CODE_SUCCESS = 'OK0000';

  /**
   * Interface configuration
   *
   * @var array
   * @see fonecta.ini
   */
  public $configuration;

  /**
   * Http client for connection
   * 
   * @var Zend_Http_Client
   */
  public $httpClient;

  /**
   * Http client configuration 
   * 
   * @var array
   */
  private $_clientConfig = array(
    'maxredirects' => 10,
    'keepalive' => true,
    'timeout'   => 300,
    'strict' 	=> true,
    'useragent' => 'Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.0.7) Gecko/2009030423 Ubuntu/8.10 (intrepid) Firefox/3.0.7 GTB5',
  );

  /*
   * Initialize client
   */
  public function __construct() { 

    $this->configuration['name_search']['service_name_id'] = variable_get('fonecta_service_name_id');
    $this->configuration['name_search']['service_category_id'] = variable_get('fonecta_service_category_id');
    $this->configuration['name_search']['service_class_id'] = variable_get('fonecta_service_class_id');
    $this->configuration['name_search']['origin_country'] = variable_get('fonecta_origin_country');
    $this->configuration['name_search']['customer_type'] = variable_get('fonecta_customer_type');
    $this->configuration['connection']['username'] = variable_get('fonecta_username');
    $this->configuration['connection']['password'] = str_replace('"', '', variable_get('fonecta_password'));
    $this->configuration['connection']['service_base_uri'] = variable_get('fonecta_base_url');

    $path = variable_get('fs_externals', FALSE);

    require_once $path['Zend_Framework'].'/Zend/Loader/Autoloader.php';

    Zend_Loader_Autoloader::getInstance();

    $this->httpClient = new Zend_Http_Client();
    $this->httpClient->setConfig($this->_clientConfig);

    $this->httpClient->setAuth(
       $this->configuration['connection']['username'], 
       $this->configuration['connection']['password'], 
       Zend_Http_Client::AUTH_BASIC);

  }

  /**
   * Query Fonecta interface for $name and filter results by record
   * customer_type=B
   *
   * @param string Name to search
   * @return array: 
   *    'key'           'value'
   *    ------------------------------------------------------
   *    header          response HEADER SXO
   *    content_header  response CONTENT_HEADER SXO
   *    content         response CONTENT SXO or empty
   */
  public function getCompanyList($name) {

    $response_str = $this->getNameSearchResponse($name);

    if($response_str === FALSE) return FALSE;

    $response_sxo = simplexml_load_string($response_str);

    $result['header'] = $response_sxo->HEADER;
    $result['content_header'] = $response_sxo->CONTENT_HEADER;

    if(!empty($response_sxo->CONTENT)) {
      foreach($response_sxo->CONTENT->ENTRY as $entry) {

        // If address publicity not set street address cannot be shown
        if(!((string)$entry->ADDRESS_PUBLICITY === 'X')) {
          unset($entry->STREET_ADDRESS);
        }

        // Filter by business type entry to remove person type entries from results 
        if((string)$entry->CUSTOMER_TYPE === 'B') {
          $result['content'][] = $entry;
        }

      }
    }

    return $result;
  }

  /**
   * Query fonecta interface by name
   *
   * @param string $name Name to search
   * @return string $response Http response body xml
   */
  public function getNameSearchResponse($name) {

    $requestParameters = $this->configuration['name_search'];
    $requestParameters['subs_name'] = $name;

    $requestUri = $this->configuration['connection']['service_base_uri'].'?';
    foreach($requestParameters as $key => $value) {
      $requestUri .= $key.'='.urlencode($value).'&';
    }
    $requestUri = trim($requestUri, '&');

    $this->httpClient->setUri($requestUri);

    $response = $this->httpClient->request(Zend_Http_Client::GET);

    if($response->getStatus() == 401) {
      return false;
    }

    $body = $response->getBody();

    return $body;
  }

  public function getTestResponse() {

    $response_str_parturi = <<<XML
<?xml version="1.0" encoding="UTF-8"?>
      <CORE_REPLY VERSION="1.1">
      <HEADER>
      <MESSAGE_CODE>OK0000</MESSAGE_CODE>
      </HEADER>
      <CONTENT_HEADER>
      <PAGING>NO_MORE</PAGING>
      <PAGE_NUMBER>1</PAGE_NUMBER>
      <MAX_PAGE>1</MAX_PAGE>
      <HIT_COUNT>1</HIT_COUNT>
      <MAX_HIT>1</MAX_HIT>
      <REPLY_TYPE>E</REPLY_TYPE>
      </CONTENT_HEADER>
      <CONTENT>
      <ENTRY>
      <ENTRY_TYPE>PETI</ENTRY_TYPE>
      <ENTRY_IDENTIFIER>404620</ENTRY_IDENTIFIER>
      <CUSTOMER_TYPE>B</CUSTOMER_TYPE>
      <SUBS_NAME>Parturi-Kampaamo Ankis HÃ¥rstudio</SUBS_NAME>
      <STREET_ADDRESS>Kauppiaankatu 18</STREET_ADDRESS>
      <ADDRESS_PUBLICITY>X</ADDRESS_PUBLICITY>
      <ADDRESS_POSTOFFICE>68600 PIETARSAARI</ADDRESS_POSTOFFICE>
      <LEVEL_2_LOCALITY>Pietarsaari</LEVEL_2_LOCALITY>
      <LONGITUDE>3287504</LONGITUDE>
      <LATITUDE>7070783</LATITUDE>
      <COMM_ADDRESS>
      <TYPE>E164</TYPE>
      <USAGE>SPEECH</USAGE>
      <CC>358</CC>
      <NDC>06</NDC>
      <SN>7230592</SN>
      <VALUE>+35867230592</VALUE>
      </COMM_ADDRESS>
      </ENTRY>
      </CONTENT>
      </CORE_REPLY>
XML;

    $response_str_parturi_many = <<<XML
<?xml version="1.0" encoding="UTF-8"?>
      <CORE_REPLY VERSION="1.1">
      <HEADER>
      <MESSAGE_CODE>OK0000</MESSAGE_CODE>
      </HEADER>
      <CONTENT_HEADER>
      <PAGING>NO_MORE</PAGING>
      <PAGE_NUMBER>1</PAGE_NUMBER>
      <MAX_PAGE>1</MAX_PAGE>
      <HIT_COUNT>1</HIT_COUNT>
      <MAX_HIT>1</MAX_HIT>
      <REPLY_TYPE>E</REPLY_TYPE>
      </CONTENT_HEADER>
      <CONTENT>
      <ENTRY>
      <ENTRY_TYPE>PETI</ENTRY_TYPE>
      <ENTRY_IDENTIFIER>404620</ENTRY_IDENTIFIER>
      <CUSTOMER_TYPE>B</CUSTOMER_TYPE>
      <SUBS_NAME>Parturi-Kampaamo Testiyritys</SUBS_NAME>
      <STREET_ADDRESS>Kauppiaankatu 18</STREET_ADDRESS>
      <ADDRESS_PUBLICITY>X</ADDRESS_PUBLICITY>
      <ADDRESS_POSTOFFICE>68600 PIETARSAARI</ADDRESS_POSTOFFICE>
      <LEVEL_2_LOCALITY>Pietarsaari</LEVEL_2_LOCALITY>
      <LONGITUDE>3287504</LONGITUDE>
      <LATITUDE>7070783</LATITUDE>
      <COMM_ADDRESS>
      <TYPE>E164</TYPE>
      <USAGE>SPEECH</USAGE>
      <CC>358</CC>
      <NDC>06</NDC>
      <SN>7230592</SN>
      <VALUE>+35867230592</VALUE>
      </COMM_ADDRESS>
      </ENTRY>
      <ENTRY>
      <ENTRY_TYPE>PETI</ENTRY_TYPE>
      <ENTRY_IDENTIFIER>404620x</ENTRY_IDENTIFIER>
      <CUSTOMER_TYPE>B</CUSTOMER_TYPE>
      <SUBS_NAME>Parturi-Kampaamo Toinen</SUBS_NAME>
      <STREET_ADDRESS>Testikatu 1</STREET_ADDRESS>
      <ADDRESS_PUBLICITY>X</ADDRESS_PUBLICITY>
      <ADDRESS_POSTOFFICE>00000 Helsinki</ADDRESS_POSTOFFICE>
      <LEVEL_2_LOCALITY>Helsinki</LEVEL_2_LOCALITY>
      <LONGITUDE>3111504</LONGITUDE>
      <LATITUDE>7011183</LATITUDE>
      <COMM_ADDRESS>
      <TYPE>E164</TYPE>
      <USAGE>SPEECH</USAGE>
      <CC>358</CC>
      <NDC>06</NDC>
      <SN>7230592</SN>
      <VALUE>+35861111192xyz</VALUE>
      </COMM_ADDRESS>
      </ENTRY>
      </CONTENT>
      </CORE_REPLY>
XML;


$response_str_err_toomany = <<<XML
<CORE_REPLY VERSION="1.1">
<HEADER>
<MESSAGE_CODE ERRMSG="Too many hits">ER9100</MESSAGE_CODE>
</HEADER>
<CONTENT_HEADER>
<PAGING>NO_SUPPORT</PAGING>
<MAX_HIT>6049</MAX_HIT>
<REPLY_TYPE>E</REPLY_TYPE>
</CONTENT_HEADER>
<CONTENT/>
</CORE_REPLY>
XML;

$response_str_err_nothing = <<<XML
<?xml version="1.0" encoding="UTF-8"?>
<CORE_REPLY VERSION="1.1">
<HEADER>
<MESSAGE_CODE ERRMSG="Nothing found">ER9090</MESSAGE_CODE>
</HEADER>
</CORE_REPLY>
XML;

$response_str_brainalliance = <<<XML
<?xml version="1.0" encoding="UTF-8"?>
<CORE_REPLY VERSION="1.1">
  <HEADER>
    <MESSAGE_CODE>OK0000</MESSAGE_CODE>
  </HEADER>
  <CONTENT_HEADER>

    <PAGING>NO_MORE</PAGING>
    <PAGE_NUMBER>1</PAGE_NUMBER>
    <MAX_PAGE>1</MAX_PAGE>
    <HIT_COUNT>5</HIT_COUNT>
    <MAX_HIT>5</MAX_HIT>
    <REPLY_TYPE>E</REPLY_TYPE>

  </CONTENT_HEADER>
  <CONTENT>
    <ENTRY>
      <ENTRY_TYPE>PETI</ENTRY_TYPE>
      <ENTRY_IDENTIFIER>818837</ENTRY_IDENTIFIER>
      <CUSTOMER_TYPE>B</CUSTOMER_TYPE>
      <SUBS_NAME>Brain Alliance Oy</SUBS_NAME>

      <STREET_ADDRESS>Hämeentie 155 C 43</STREET_ADDRESS>
      <ADDRESS_PUBLICITY>X</ADDRESS_PUBLICITY>
      <ADDRESS_POSTOFFICE>00560 HELSINKI</ADDRESS_POSTOFFICE>
      <LEVEL_2_LOCALITY>Helsinki</LEVEL_2_LOCALITY>
      <LONGITUDE>3388130</LONGITUDE>
      <LATITUDE>6679624</LATITUDE>

      <COMM_ADDRESS>
        <TYPE>E164</TYPE>
        <USAGE>SPEECH</USAGE>
        <CC>358</CC>
        <NDC>010</NDC>
        <SN>3466440</SN>

        <VALUE>+358103466440</VALUE>
      </COMM_ADDRESS>
    </ENTRY>
    <ENTRY>
      <ENTRY_TYPE>VASI</ENTRY_TYPE>
      <ENTRY_IDENTIFIER>F49P0400169343</ENTRY_IDENTIFIER>
      <ENTRY_LEVEL>1</ENTRY_LEVEL>

      <SUBS_NAME>BRAIN ALLIANCE SOLUTIONS OY</SUBS_NAME>
      <STREET_ADDRESS>HÄMEENTIE 155 C 43</STREET_ADDRESS>
      <ADDRESS_POSTOFFICE>00560 HELSINKI</ADDRESS_POSTOFFICE>
      <LEVEL_2_LOCALITY>Helsinki</LEVEL_2_LOCALITY>
      <OPERATOR_ID>49</OPERATOR_ID>
      <OPERATOR_NAME>Sonera</OPERATOR_NAME>

      <COMM_ADDRESS>
        <TYPE>MOB</TYPE>
        <USAGE>SPEECH</USAGE>
        <CC>358</CC>
        <NDC>0400</NDC>
        <SN>169343</SN>

        <VALUE>+358400169343</VALUE>
      </COMM_ADDRESS>
    </ENTRY>
    <ENTRY>
      <ENTRY_TYPE>VASI</ENTRY_TYPE>
      <ENTRY_IDENTIFIER>F56P0451230891</ENTRY_IDENTIFIER>
      <ENTRY_LEVEL>1</ENTRY_LEVEL>

      <SUBS_NAME>Hassinen Jukka</SUBS_NAME>
      <STREET_ADDRESS>Särkiniementie 5 A 61</STREET_ADDRESS>
      <ADDRESS_POSTOFFICE>00210 HELSINKI</ADDRESS_POSTOFFICE>
      <LEVEL_2_LOCALITY>Helsinki</LEVEL_2_LOCALITY>
      <OPERATOR_ID>56</OPERATOR_ID>
      <OPERATOR_NAME>Saunalahti</OPERATOR_NAME>

      <COMM_ADDRESS>
        <TYPE>MOB</TYPE>
        <USAGE>SPEECH</USAGE>
        <CC>358</CC>
        <NDC>045</NDC>
        <SN>1230891</SN>

        <VALUE>+358451230891</VALUE>
      </COMM_ADDRESS>
      <ENTRY>
        <ENTRY_LEVEL>2</ENTRY_LEVEL>
        <SUBS_NAME>Brain Alliance Solutions Oy</SUBS_NAME>
        <STREET_ADDRESS>Särkiniementie 5 A 61</STREET_ADDRESS>
        <ADDRESS_POSTOFFICE>00210 HELSINKI</ADDRESS_POSTOFFICE>

        <LEVEL_2_LOCALITY>Helsinki</LEVEL_2_LOCALITY>
        <OPERATOR_ID>56</OPERATOR_ID>
        <OPERATOR_NAME>Saunalahti</OPERATOR_NAME>
      </ENTRY>
    </ENTRY>
    <ENTRY>
      <ENTRY_TYPE>VASI</ENTRY_TYPE>

      <ENTRY_IDENTIFIER>F56P0451230892</ENTRY_IDENTIFIER>
      <ENTRY_LEVEL>1</ENTRY_LEVEL>
      <SUBS_NAME>Hassinen Jukka</SUBS_NAME>
      <STREET_ADDRESS>Särkiniementie 5 A 61</STREET_ADDRESS>
      <ADDRESS_POSTOFFICE>00210 HELSINKI</ADDRESS_POSTOFFICE>
      <LEVEL_2_LOCALITY>Helsinki</LEVEL_2_LOCALITY>

      <OPERATOR_ID>56</OPERATOR_ID>
      <OPERATOR_NAME>Saunalahti</OPERATOR_NAME>
      <COMM_ADDRESS>
        <TYPE>MOB</TYPE>
        <USAGE>SPEECH</USAGE>
        <CC>358</CC>

        <NDC>045</NDC>
        <SN>1230892</SN>
        <VALUE>+358451230892</VALUE>
      </COMM_ADDRESS>
      <ENTRY>
        <ENTRY_LEVEL>2</ENTRY_LEVEL>
        <SUBS_NAME>Brain Alliance Solutions Oy</SUBS_NAME>

        <STREET_ADDRESS>Särkiniementie 5 A 61</STREET_ADDRESS>
        <ADDRESS_POSTOFFICE>00210 HELSINKI</ADDRESS_POSTOFFICE>
        <LEVEL_2_LOCALITY>Helsinki</LEVEL_2_LOCALITY>
        <OPERATOR_ID>56</OPERATOR_ID>
        <OPERATOR_NAME>Saunalahti</OPERATOR_NAME>
      </ENTRY>

    </ENTRY>
    <ENTRY>
      <ENTRY_TYPE>VASI</ENTRY_TYPE>
      <ENTRY_IDENTIFIER>F56P0456302325</ENTRY_IDENTIFIER>
      <ENTRY_LEVEL>1</ENTRY_LEVEL>
      <SUBS_NAME>Hassinen Jukka</SUBS_NAME>
      <STREET_ADDRESS>Särkiniementie 5 A 61</STREET_ADDRESS>

      <ADDRESS_POSTOFFICE>00210 HELSINKI</ADDRESS_POSTOFFICE>
      <LEVEL_2_LOCALITY>Helsinki</LEVEL_2_LOCALITY>
      <OPERATOR_ID>56</OPERATOR_ID>
      <OPERATOR_NAME>Saunalahti</OPERATOR_NAME>
      <COMM_ADDRESS>
        <TYPE>MOB</TYPE>

        <USAGE>SPEECH</USAGE>
        <CC>358</CC>
        <NDC>045</NDC>
        <SN>6302325</SN>
        <VALUE>+358456302325</VALUE>
      </COMM_ADDRESS>

      <ENTRY>
        <ENTRY_LEVEL>2</ENTRY_LEVEL>
        <SUBS_NAME>Brain Alliance Solutions Oy</SUBS_NAME>
        <STREET_ADDRESS>Särkiniementie 5 A 61</STREET_ADDRESS>
        <ADDRESS_POSTOFFICE>00210 HELSINKI</ADDRESS_POSTOFFICE>
        <LEVEL_2_LOCALITY>Helsinki</LEVEL_2_LOCALITY>

        <OPERATOR_ID>56</OPERATOR_ID>
        <OPERATOR_NAME>Saunalahti</OPERATOR_NAME>
      </ENTRY>
    </ENTRY>
  </CONTENT>
</CORE_REPLY>
XML;

    return $response_str_brainalliance;
  }

}

